# Effects
SAD_MOMENT_GOOGLE_DRIVE_DOWNLOAD_URL = 'https://drive.google.com/file/d/17rw299LIMTEciS7WDgFbIEZ7DckpHsC7/view?usp=sharing'
PHOTO_GOOGLE_DRIVE_DOWNLOAD_URL = 'https://drive.google.com/file/d/1xr6x_yeU9SF5guB6KA9cLqbmVUwxBMi8/view?usp=sharing'

# Manim
WHOOSH_TITLE_DRIVE_DOWNLOAD_URL = 'https://drive.google.com/file/d/1WPS8uWB1LTuzPzxQ2Zcp1FpwvLM3fhM5/view?usp=sharing'
MINECRAFT_ITEM_DROP_DRIVE_DOWNLOAD_URL = 'https://drive.google.com/file/d/1Dzeb6Qae4UdpmuA6U9d6t3MXnvhzukzg/view?usp=sharing'